beta.spe=-3; eta_v0 <- -0.886; alpha_0<-5.3

library(nnet)
library(splines)

library(MASS)

source(file="myfunctions_Mb.R")
#creat dataset
n<-24000
mu.X<-11
sd.X<-1
set.seed(1)
### Step 2: generate X
dat<-data.frame(rnorm(n,mean=mu.X,sd=sd.X))
colnames(dat)<-c("X")
summary(dat$X)
dat$XYRS<-dat$X
dat$X<-cut(dat$X,c(quantile(dat$X,probs = c(0,0.25,0.5,0.75)), max(dat$X)+1), right = FALSE)
dat$X2<-dat$X==levels(dat$X)[2]
dat$X3<-dat$X==levels(dat$X)[3]
dat$X4<-dat$X==levels(dat$X)[4]


### Step 1: generate Z
dat$Z<-c(rep(0,n/3),rep(1,2*n/3))

#Step 3: generate Ytau1
mu_0<-c(4.4,-0.23,-0.46, -0.69)
X<-cbind(rep(1,n),dat$X2, dat$X3, dat$X4)
R<-X%*%mu_0
theta<- exp(R)/(1+exp(R))
dat$Ytau1<-rbinom(n,1,1-theta)
sum(dat$Ytau1==0)/n #need to be around 0.98
rm(R, theta)

### Step 4: generate S(1)
dat$S1<-NA

gammax<-c(2.4,-0.4,0.6,0.9)
sdx<-0.68

X<-cbind(rep(1,sum(dat$Ytau1==0)),dat$X2[dat$Ytau1==0], dat$X3[dat$Ytau1==0], dat$X4[dat$Ytau1==0])
mu<-X%*%gammax

gamma.true<-c(gammax,sdx)

dat$S1[dat$Ytau1==0]<- rnorm(sum(dat$Ytau1==0),mean=mu,sd=sdx)
dat$S1full<-dat$S1
summary(dat$S1)
cor(dat$S1[dat$Ytau1==0], dat$XYRS[dat$Ytau1==0]) #=0.45


## (M4b)
g_v_EAAR<-function(s_1, x2,x3,x4,eta_v0,eta_vs,eta_vx2,eta_vx3,eta_vx4){
  X<-cbind(rep(1,length(s_1)),s_1, x2,x3,x4)
  R<-X%*%c(eta_v0,eta_vs,eta_vx2,eta_vx3,eta_vx4)
  R<-pnorm(R)
  return(R)
}

## (A4)
w<-function(s_1,x2,x3,x4,y,alpha_0,alpha_s,alpha_x,beta){
  par<-c(alpha_0,alpha_s,alpha_x,beta)
  X<-cbind(rep(1,length(s_1)),s_1,x2,x3,x4,y)
  R<-X%*%par
  w<-exp(R)/(1+exp(R))
  return(w)
}


g_v<-function(s_1, x2,x3,x4,eta_v0,eta_vs,eta_vx2,eta_vx3,eta_vx4,alpha_0,alpha_s,alpha_x,beta){
  out<-(1/w(s_1,x2,x3,x4,1,alpha_0,alpha_s,alpha_x,beta))*g_v_EAAR(s_1, x2,x3,x4,eta_v0,eta_vs,eta_vx2,eta_vx3,eta_vx4)/((1/w(s_1,x2,x3,x4,1,alpha_0,alpha_s,alpha_x,beta))*g_v_EAAR(s_1, x2,x3,x4,eta_v0,eta_vs,eta_vx2,eta_vx3,eta_vx4)+(1/w(s_1,x2,x3,x4,0,alpha_0,alpha_s,alpha_x,beta))*(1-g_v_EAAR(s_1, x2,x3,x4,eta_v0,eta_vs,eta_vx2,eta_vx3,eta_vx4)))
  return(out)
}
eta_p0 <- -0.78


eta_ps <- -0.22
eta_px2 <- eta_vx2 <- -0.13
eta_px3 <- eta_vx3 <- -0.19
eta_px4 <- eta_vx4 <- -0.31
eta_vs <- -0.465

alpha_s<- -log(2) #under vaccine is at risk, S(1) is high means staying at risk under vaccine is mainly becuase of vaccine protection, so staying at risk under placebo is low.
alpha_x<-c(log(1.1),log(1.3),log(1.7)) #the older they are, the more likely they stay at risk


### Step 5: generate Y1
dat$Y1<-NA
dat$Y1[dat$Ytau1==1]<-1

R<-g_v(dat$S1,dat$X2,dat$X3,dat$X4,eta_v0,eta_vs,eta_vx2,eta_vx3,eta_vx4,alpha_0,alpha_s,alpha_x,beta)
summary(R[dat$Ytau1==0])

dat$Y1[dat$Ytau1==0]<-rbinom(sum(dat$Ytau1==0),1,R[dat$Ytau1==0])
table(dat$Y1)
#to decrease, increase alpha_0 or decrease eta_v0
sum(dat$Y1[dat$Ytau1==0])/length(dat$Y1[dat$Ytau1==0])  #should be about 0.026
rm(R)

### Step 6: generate Ytau0
dat$Ytau0<-NA
dat$Ytau0[dat$Ytau1==1]<-1

myw<-w(dat$S1,dat$X2,dat$X3,dat$X4, dat$Y1,alpha_0,alpha_s,alpha_x,beta)

dat$Ytau0[dat$Ytau1==0]<-rbinom(sum(dat$Ytau1==0),1,1-myw[dat$Ytau1==0])
# to decrease this, decrease alpha_0 or 
sum(dat$Ytau0==0)/sum(dat$Ytau1==0) #should be around 0.96

### Step 7: generate Y0
dat$Y0<-NA
dat$Y0[dat$Ytau0==1]<-1

beta_0<-c(eta_p0, eta_ps,eta_px2,eta_px3, eta_px4)
X<-cbind(rep(1,n),dat$S1,dat$X2,dat$X3, dat$X4)
R<-X%*%beta_0
R<-pnorm(R)
summary(R[dat$Ytau0==0]) #should be about 0.07

dat$Y0[dat$Ytau0==0]<-rbinom(sum(dat$Ytau0==0),1,R[dat$Ytau0==0])
sum(dat$Y0[dat$Ytau0==0]==1)/sum(dat$Ytau0==0)  #should be about 0.07
rm(X,R)

### Step 8 and 9: Only record observables
dat$S1<-ifelse(dat$Z==1,dat$S1,NA)
dat$Y<-ifelse(dat$Z==1,dat$Y1,dat$Y0)
table(dat$Y)
dat$Ytau<-ifelse(dat$Z==1,dat$Ytau1,dat$Ytau0)
dat$S1[dat$Ytau==1]<-NA

### Step 10: two phase sampling
dat$Immuno<-NA

dat$Immuno[dat$Ytau==0]<-rbinom(sum(dat$Ytau==0),1,0.2)
dat$S1<-ifelse(dat$Immuno==1|dat$Y==1,dat$S1,NA)
table(is.na(dat$S1))

data<-dat[,c("Z","X","X2","X3","X4","S1","Y","Ytau")]

   Su<-c(0,0.5,1,1.5,2,2.5,3,3.5,4,4.5,5)
   eta_v_true<-c(eta_v0,-0.465, -0.13, -0.19, -0.31)
    eta_p_true<-c(-0.78, -0.22, -0.13, -0.19, -0.31)
    alpha_true<-c(alpha_0, -log(2), log(1.1),log(1.3),log(1.7))
    par.true<-c(alpha_true,eta_v_true, eta_p_true)
    
    
    beta.spe=beta.spe.seq[iter.betaspe]
    
    dat$offset=beta.spe*dat$Y
    
    fit=glm(I(Ytau0==0)~S1full+X2+X3+X4,offset=offset,data=dat[dat$Z==1 & dat$Ytau==0,],family=binomial(link=logit))
    
    xi<-rep(1,nrow(data)) ### perturbation weight, all 1 if no perturbation.
    #xi<-rexp(nrow(data),rate=1)    
    
    fit2=glm(I(Ytau0==0)~S1full+X2+X3+X4,offset=offset,data=dat[dat$Z==1 & dat$Ytau==0,],family=binomial(link=logit),weights=xi[dat$Z==1 & dat$Ytau==0]) 
             

 
    par.start<-par.true
    par.start[1:5]<-fit$coef
        
    par.start.2=par.true
    par.start.2[1:5]<-fit2$coef
        
    results<- getVE.sen(data=data,beta=beta.spe,par.start=par.start,par.true=par.true,gamma.true=gamma.true,beta.true=beta,Su=Su,xi=xi,par.start.2=par.start.2)

    ## parameter estimate
      results$gamma
      results$mu
      results$alpha
      results$eta_v
      results$eta_p
       
        
      ## VE estimate  
      results$VE
      results$VE.2
   
