#create duumy variables for X
getDummy.New<-function(dat){
  for (i in 2:(length(levels(dat$X)))){
    dat[[paste0("X",i)]]<-dat$X==levels(dat$X)[i]
  }
  return(dat)
}

R.probit<-function(Z,S1,X,eta){
  Risk<-NULL
  datin=data.frame(Z=Z,S1=S1,X=X)
  datin=getDummy.New(datin)
  if (length(S1)>0) {
    Xmatrix<-data.matrix(cbind(1,datin$Z,datin$S1,datin$S1*datin$Z,datin[,grep("^[X]", names(datin), value=TRUE)[-1]],datin[,grep("^[X]", names(datin), value=TRUE)[-1]]*datin$Z))
    RR<-Xmatrix%*%as.matrix(eta)
    Risk<-pnorm(RR)
  } else Risk<-NULL
  return(Risk)
}

#### negative log-likelihood for observing Y^tau(1)=0
#### mu: parameter
#### xi: individiaul weight
logl1<-function(mu,data,xi){
  dat<-data[data$Z==1,]
  X<-data.matrix(cbind(rep(1,nrow(dat)),dat[,grep("^[X]", names(dat), value=TRUE)[-1]]))
  theta<-exp(X%*%mu)/(1+exp(X%*%mu))
  ff<-ifelse(dat$Ytau==0, theta, 1-theta)
  ff<-log(ff)*xi[data$Z==1]
  return(-sum(ff))
}


### negative log-likelihood for S(1)|X,Y^tau(1)=0 
logl2<-function(gamma,prob0,prob1,data,xi){
  dat<-data[!is.na(data$S1),]
  X<-data.matrix(cbind(rep(1,nrow(dat)),dat[,grep("^[X]", names(dat), value=TRUE)[-1]]))
  tmp<-head(gamma,length(gamma)-1)
  mymean<-X%*%tmp
  ff<-dnorm(dat$S1,mean = mymean, sd= tail(gamma,1))
  ff<-ifelse(dat$Y==1,ff*(1/prob1),ff*(1/prob0))
  ff<-log(ff)*xi[!is.na(data$S1)]
  return(-sum(ff))
}

### construct estimated likelihood 
like<-function(beta,mu,gamma,par,data,xi){
  #alpha=c(alpha_0,alpha_s,alpha_x), length=2+length(levels(data$X))-1
  alpha<-par[1:(2+length(levels(data$X))-1)]  # (alpha_0,alpha_s,alpha_x)
  eta<-par[(2+length(levels(data$X))):length(par)] 
  #eta_v<-c(eta[1]+eta[2],eta[3]+eta[4],eta[5]+eta[8] , eta[6]+eta[9], eta[7]+eta[10])
  #eta_p<-c(eta[1],eta[3],eta[5] , eta[6], eta[7])
  ans <- 0
  data$xi<-xi
  
  #Ytau=0, Z=1, delta=1
  dat<-data[data$Ytau==0&data$Z==1&!is.na(data$S1),] #3506
  piy0z1<-NULL
  for (i in 1:length(levels(data$X))){
    piy0z1<-c(piy0z1,nrow(data[data$Ytau==0 & data$Y==0 & data$Z==1 &data$X==levels(data$X)[i] & !is.na(data$S1),])/nrow(data[data$Ytau==0 & data$Y==0 & data$Z==1 &data$X==levels(data$X)[i],]) )
  }
  
  eqn12<-rep(NA,nrow(dat))
  eqn12<-piy0z1[as.numeric(dat$X)]
  eqn12[dat$Y==1]<-1  # can be left out
  
  ### g_v(S_i(1),X_i)^Y_i(1-g_v(S_i(1),X_i))^(1-Y_i)
  X<-data.matrix(cbind(rep(1,nrow(dat)),dat$Z,dat$S1,dat$S1*dat$Z,dat[,grep("^[X]", names(dat), value=TRUE)[-1]],dat[,grep("^[X]", names(dat), value=TRUE)[-1]]*dat$Z))
  eqn13<-ifelse(dat$Y==1,pnorm(X%*%eta), 1-pnorm(X%*%eta))
  
  ### f_v(S_i(1)|X_i)
  X<-data.matrix(cbind(rep(1,nrow(dat)),dat[,grep("^[X]", names(dat), value=TRUE)[-1]]))
  mean<-X%*%head(gamma,-1)
  eqn14<-dnorm(dat$S1,mean=mean,sd=tail(gamma,1))  ### can be left out
  
  ### theta_v(X_i)
  eqn15<-exp(X%*%mu)/(1+exp(X%*%mu))  # can be left out
  
  ####
  #ff<-eqn12*eqn13*eqn14*eqn15
  ff<-eqn13
  ff<-log(ff)*dat$xi
  ans<-ans+sum(ff)
  
  
  # Ytau=0, Z=1, delta=0
  dat<-data[data$Ytau==0&data$Z==1&is.na(data$S1),] #12193
  
  eqn16<-rep(NA,nrow(dat))
  eqn16<-1-piy0z1[as.numeric(dat$X)]  ### can be left out
  
  
  
  myeta4<-0; myeta5<-0; mymean<-gamma[1]
  
   integrand<-function(s){
      out<-(1-pnorm(eta[1]+eta[2]+eta[3]*s+eta[4]*s+myeta4+myeta5))*dnorm(s,mean=mymean,sd=tail(gamma,1))
     return(out)
    }
  myterm<-integrate(integrand,lower=-Inf,upper=Inf)$value
  for(i in 2:length(levels(dat$X))){
    myeta4<-eta[length(eta)-2*(length(levels(dat$X))-1)+i-1]
    myeta5<-eta[length(eta)-(length(levels(dat$X))-1)+i-1]
    mymean<-gamma[1]+gamma[i]
    
    #eqn17 takes on one value for each of X levels, note here we use condiction normal distribution for S(1)|Z=1,Ytau=0. Alternatively one could use semiparametric or
    ### non-parametric model for estimation
    integrand<-function(s){
      out<-(1-pnorm(eta[1]+eta[2]+eta[3]*s+eta[4]*s+myeta4+myeta5))*dnorm(s,mean=mymean,sd=tail(gamma,1))
     return(out)
    }
    
    myterm<-c(myterm, integrate(integrand,lower=-Inf,upper=Inf)$value)
  }
  
  ### match proper value for each observation based on corresponding X value
  eqn17<-myterm[as.numeric(dat$X)]
  
  X<-data.matrix(cbind(rep(1,nrow(dat)),dat[,grep("^[X]", names(dat), value=TRUE)[-1]]))
  eqn18<-exp(X%*%mu)/(1+exp(X%*%mu))  ### can be left out
  
  #ff<-eqn16*eqn17*eqn18
  ff<-eqn17
  ff<-log(ff)*dat$xi
  ans<-ans+sum(ff)
  
  #Ytau=0, Z=0
  
  dat<-data[data$Ytau==0 & data$Z==0,] #7536
  eqn19<-1 # can be left out
  
  eta_v<-c(eta[1]+eta[2],eta[3]+eta[4],eta[5]+eta[8] , eta[6]+eta[9], eta[7]+eta[10])
  eta_p<-c(eta[1],eta[3],eta[5] , eta[6], eta[7])
  
  #eqn20 takes on 1 value for each X and each Y
  #need to specify myx
  
  ### get gvEAAR.star from W() and gv() under Ma
  gvEAAR<-function(s,myx){
    R<- alpha[1]+alpha[2]*s+alpha[3]*myx[1]+alpha[4]*myx[2]+alpha[5]*myx[3]+beta*1  # for y=1
    w_y1<-exp(R)/(1+exp(R))
    rm(R)
    
    R<- alpha[1]+alpha[2]*s+alpha[3]*myx[1]+alpha[4]*myx[2]+alpha[5]*myx[3]+beta*0 # for y=0
    w_y0<-exp(R)/(1+exp(R))
    rm(R)
    
    R<-eta_v[1]+eta_v[2]*s+eta_v[3]*myx[1]+eta_v[4]*myx[2]+eta_v[5]*myx[3]
    g_v<-pnorm(R)
    rm(R)
    
    gvEAAR <- w_y1*g_v/(w_y1*g_v+w_y0*(1-g_v))
    gvEAAR[gvEAAR==0]<-2e-10
    return(gvEAAR)
  }
  
  ### get gpEAAR
  g_p<-function(s,myx){
    R<-eta_p[1]+eta_p[2]*s+eta_p[3]*myx[1]+eta_p[4]*myx[2]+eta_p[5]*myx[3]
    return(pnorm(R))
  }
  
  ### numerator for f_p^a(s_1|x) under Ma 
  num<-function(s,myx){
    R<- alpha[1]+alpha[2]*s+alpha[3]*myx[1]+alpha[4]*myx[2]+alpha[5]*myx[3]+beta*1
    w_y1<-exp(R)/(1+exp(R))
    rm(R)
    
    R<- alpha[1]+alpha[2]*s+alpha[3]*myx[1]+alpha[4]*myx[2]+alpha[5]*myx[3]+beta*0
    w_y0<-exp(R)/(1+exp(R))
    rm(R)
    
    R<-eta_v[1]+eta_v[2]*s+eta_v[3]*myx[1]+eta_v[4]*myx[2]+eta_v[5]*myx[3]
    g_v<-pnorm(R)
    rm(R)
    
    X<-c(1,myx)
    tmp<-head(gamma,length(gamma)-1)
    mymean<-sum(X*tmp)
    f<-dnorm(s,mean = mymean, sd= tail(gamma,1))
    rm(X)
    
    mygvEAAR <- gvEAAR(s,myx)
    num <- w_y1*g_v*f/mygvEAAR
    return(num)
  }
  
  ### compute f_p^a(s_1|x)
  f_p<-function(s,myx){
    den<-integrate(num,lower=-100,upper=100,myx=myx)$value
    return(num(s,myx = myx)/den)
  }
  
  #need to specify myx and myy
  integrand<-function(s,myx,myy){
    out<- (g_p(s,myx)^myy)*((1-g_p(s,myx))^(1-myy))*f_p(s,myx)
    return(out)
  }
  
  myy<-1
  myx<-c(0,0,0) 
  myterm.y1<-integrate(integrand,lower=-100,upper=100,myx=myx,myy=myy)$value
  
  myx<-c(1,0,0) 
  myterm.y1<-c(myterm.y1,integrate(integrand,lower=-100,upper=100,myx=myx,myy=myy)$value)
  
  myx<-c(0,1,0) 
  myterm.y1<-c(myterm.y1,integrate(integrand,lower=-100,upper=100,myx=myx,myy=myy)$value)
  
  myx<-c(0,0,1) 
  myterm.y1<-c(myterm.y1,integrate(integrand,lower=-100,upper=100,myx=myx,myy=myy)$value)
  
  myy<-0
  myx<-c(0,0,0) 
  myterm.y0<-integrate(integrand,lower=-100,upper=100,myx=myx,myy=myy)$value
  
  myx<-c(1,0,0) 
  myterm.y0<-c(myterm.y0,integrate(integrand,lower=-100,upper=100,myx=myx,myy=myy)$value)
  
  myx<-c(0,1,0) 
  myterm.y0<-c(myterm.y0,integrate(integrand,lower=-100,upper=100,myx=myx,myy=myy)$value)
  
  myx<-c(0,0,1) 
  myterm.y0<-c(myterm.y0,integrate(integrand,lower=-100,upper=100,myx=myx,myy=myy)$value)
  
  #eqn20 takes on 1 value for each X and each Y: g_p^EAAR(s_1,X)^Y(1-g_p^EAAR(s_1,X)^(1-Y)
  eqn20<-rep(NA, nrow(dat))
  eqn20[dat$Y==1]<-myterm.y1[as.numeric(dat$X[dat$Y==1])]
  eqn20[dat$Y==0]<-myterm.y0[as.numeric(dat$X[dat$Y==0])]
  
  #eqn21 takes on 1 values for each X
  myx<-c(0,0,0) 
  X<-c(1,myx)
  theta_v<-exp(sum(X*mu))/(1+exp(sum(X*mu)))
  myterm.theta.v<-theta_v
  myterm.theta.p<- theta_v*integrate(num,lower=-100,upper=100,myx=myx)$value
  
  myx<-c(1,0,0) 
  X<-c(1,myx)
  theta_v<-exp(sum(X*mu))/(1+exp(sum(X*mu)))
  myterm.theta.v<-c(myterm.theta.v, theta_v)
  myterm.theta.p<- c(myterm.theta.p, theta_v*integrate(num,lower=-100,upper=100,myx=myx)$value)
  
  myx<-c(0,1,0) 
  X<-c(1,myx)
  theta_v<-exp(sum(X*mu))/(1+exp(sum(X*mu)))
  myterm.theta.v<-c(myterm.theta.v, theta_v)
  myterm.theta.p<- c(myterm.theta.p, theta_v*integrate(num,lower=-100,upper=100,myx=myx)$value)
  
  myx<-c(0,0,1) 
  X<-c(1,myx)
  theta_v<-exp(sum(X*mu))/(1+exp(sum(X*mu)))
  myterm.theta.v<-c(myterm.theta.v, theta_v)
  myterm.theta.p<- c(myterm.theta.p, theta_v*integrate(num,lower=-100,upper=100,myx=myx)$value)
  
  eqn21<-myterm.theta.p[as.numeric(dat$X)]
  
  #ff<-eqn19*eqn20*eqn21
  ff<-eqn20*eqn21
  ff<-log(ff)*dat$xi
  ans<-ans+sum(ff)
  
  #Ytau=1
  dat<-data[data$Ytau==1,]
  eqn22<-1 # can be left out
  eqn23<-rep(NA, nrow(dat))
  eqn23[dat$Z==1]<-1-myterm.theta.v[as.numeric(dat$X[dat$Z==1])]  ### can be left out
  eqn23[dat$Z==0]<-1-myterm.theta.p[as.numeric(dat$X[dat$Z==0])]
  
  ff<-eqn22*eqn23
  ff<-log(ff)*dat$xi
  ans<-ans+sum(ff)
  
  #(Negative) log-likelihood function
  ans<- -ans
  return(ans)
}

getVE<-function(data,beta,par.start,par.true,gamma.true,Su,xi){
  #get nuisance parameters:
  #estimate \mu
  #mu.start <- c(4.6,rep(0,length(levels(data$X))-1))
  
  X.mu<-data.matrix(cbind(rep(1,nrow(data)),data[,grep("^[X]", names(data), value=TRUE)[-1]]))
  
  mu.start<-glm.fit(X.mu[data$Z==1,],1-data[data$Z==1,]$Ytau,weights=xi[data$Z==1], family=binomial(link="logit"))$coef
  #mu <- optim(par=mu.start, fn=logl1,data=data, xi=xi)$par
  mu <- mu.start
  rm(mu.start)
  
  #Estimate gamma
  #case sampling prob
  prob1<-sum( xi[ data$Ytau==0&data$Y==1&data$Z==1&!is.na(data$S1) ] )/sum(xi[data$Ytau==0&data$Z==1&data$Y==1])
  
  #control vaccinees sampling prob
  prob0<-sum(xi[data$Ytau==0&data$Y==0&data$Z==1&!is.na(data$S1)])/sum(xi[data$Ytau==0&data$Z==1&data$Y==0])
  
  #gamma.start <- c(2.4,rep(0,length(levels(data$X))-1), 0.58)
  
  ff<-ifelse(data$Y==1,(1/prob1),(1/prob0))
  kk=glm.fit(X.mu[data$Z==1 & data$Ytau==0 & !is.na(data$S),],data$S[data$Z==1 & data$Ytau==0 & !is.na(data$S)],weights=(ff*xi)[data$Z==1 & data$Ytau==0 & !is.na(data$S)],family=gaussian(link=identity))
  
  gamma.start<-c(kk$coef, sqrt(sum(kk$residuals^2*kk$weights)/sum(kk$weights)-sum((kk$residual*kk$weights)/sum(kk$weights))^2))
  #gamma <- optim(par=gamma.start, fn=logl2,prob0=prob0, prob1=prob1,data=data, xi=xi)$par
  gamma<-gamma.start
  rm(gamma.start)
  
  ### estimate eta
  out <- optim(par=par.start, fn=like,beta=beta,mu=mu,gamma=gamma,data=data, xi=xi )$par
  alpha<-out[1:(2+length(levels(data$X))-1)]
  eta<-tail(out, 4+ 2*(length(levels(data$X))-1))
  eta_v<-c(eta[1]+eta[2],eta[3]+eta[4],eta[5]+eta[8] , eta[6]+eta[9], eta[7]+eta[10])
  eta_p<-c(eta[1],eta[3],eta[5] , eta[6], eta[7])
  
  #######estimate X|S(1), Y\tau(0)=Y\tau(1)=0:
  Xu<-sort(unique(data$X[!is.na(data$X)]))
  datnew<-data.frame(S1=Su)
  
  
  #need to specify myx
  gvEAAR<-function(s,myx,alpha,beta){
    R<- alpha[1]+alpha[2]*s+alpha[3]*myx[1]+alpha[4]*myx[2]+alpha[5]*myx[3]+beta*1
    w_y1<-exp(R)/(1+exp(R))
    rm(R)
    
    R<- alpha[1]+alpha[2]*s+alpha[3]*myx[1]+alpha[4]*myx[2]+alpha[5]*myx[3]+beta*0
    w_y0<-exp(R)/(1+exp(R))
    rm(R)
    
    R<-eta_v[1]+eta_v[2]*s+eta_v[3]*myx[1]+eta_v[4]*myx[2]+eta_v[5]*myx[3]
    g_v<-pnorm(R)
    rm(R)
    
    gvEAAR <- w_y1*g_v/(w_y1*g_v+w_y0*(1-g_v))
    gvEAAR[gvEAAR==0]<-2e-10
    return(gvEAAR)
  }
  
  g_v<-function(s,myx,eta_v){
    R<-eta_v[1]+eta_v[2]*s+eta_v[3]*myx[1]+eta_v[4]*myx[2]+eta_v[5]*myx[3]
    return(pnorm(R))
  }
  
  g_p<-function(s,myx,eta_p){
    R<-eta_p[1]+eta_p[2]*s+eta_p[3]*myx[1]+eta_p[4]*myx[2]+eta_p[5]*myx[3]
    return(pnorm(R))
  }
  
  #P(Y^tau(0)=0|X=x, s_1, Y^tau(1)=0)
  f.Yyao0<-function(myx,s,alpha,beta,eta_v){
    R<- alpha[1]+alpha[2]*s+alpha[3]*myx[1]+alpha[4]*myx[2]+alpha[5]*myx[3]+beta*1
    w_y1<-exp(R)/(1+exp(R))
    rm(R)
    
    R<- alpha[1]+alpha[2]*s+alpha[3]*myx[1]+alpha[4]*myx[2]+alpha[5]*myx[3]+beta*0
    w_y0<-exp(R)/(1+exp(R))
    rm(R)
    
    return(w_y1*g_v(s,myx,eta_v)+w_y0*(1-g_v(s,myx,eta_v)))
  }
  
  ### compute P(X|S(1),Y^tau(0)=Y^tau(1)=0)
  getfx.s2<-function(fx.s,alpha,beta,eta_v){
  ## fx.s input is P(X|S(1),Y^tau(1)=0)
  
      fx.s2<-cbind(f.Yyao0(c(0,0,0),Su,alpha,beta,eta_v),f.Yyao0(c(1,0,0),Su,alpha,beta,eta_v),f.Yyao0(c(0,1,0),Su,alpha,beta,eta_v),f.Yyao0(c(0,0,1),Su,alpha,beta,eta_v))
      fx.s2<- fx.s*fx.s2
      den<-apply(fx.s2, 1, sum)
      den<-cbind(den,den,den,den)
      fx.s2<-fx.s2/den
      return(fx.s2)
  }
  
  # compute P(X=x|S(1),Y^tau(1)=0)
  #fX<-table(data$X)/sum(!is.na(data$X))
  data.delta1<-data[!is.na(data$S1),]
  
  #weights of sampling S1 
  data.delta1$weights<-rep(1,nrow(data.delta1))
  data.delta1$weights[data.delta1$Y==0]<-prob0
  
  wts <- 1/data.delta1$weights 
  wts<-wts*xi[!is.na(data$S1)]    #purturbation for estimating multinomial parameters
  
  fit=multinom(X~ns(S1,knots=quantile(Su,c(.25,.5,.75)),Boundary.knots=range(Su)),data=data.delta1,weights=wts)
  
  fx.s=predict(fit,datnew,type='probs')
  
  
  ## alternative way to get P(X=x|S(1),Y^tau(1)=0
  
  fv.Su<-function(myx,s,gamma){
    R<- gamma[1]+gamma[2]*myx[1]+gamma[3]*myx[2]+gamma[4]*myx[3]
    return(dnorm(s,R,gamma[5]))
  }
    
  fx.ss=cbind(fv.Su(c(0,0,0),Su,gamma),fv.Su(c(1,0,0),Su,gamma),fv.Su(c(0,1,0),Su,gamma),fv.Su(c(0,0,1),Su,gamma))
  #fX<-table(data[data$Z==1 & data$Ytau==0,]$X)/sum(!is.na(data[data$Z==1 & data$Ytau==0,]$X))
  
  fX<-tapply(xi[data$Z==1 & data$Ytau==0],data[data$Z==1 & data$Ytau==0,]$X,sum)/sum(xi[data$Z==1 & data$Ytau==0])
  
  
  
  fx.snew<-t(t(fx.ss)*as.numeric(fX))/apply(t(fx.ss)*as.numeric(fX),2,sum)
  
  
  fx.s2<-getfx.s2(fx.s,alpha,beta,eta_v)
  fx.s2new<-getfx.s2(fx.snew,alpha,beta,eta_v)
  PY1.SZ0<-PY1.SZ1<-rep(0,length(Su))
  for (i in 1:length(Xu)){
    if(i==1){myx<-c(0,0,0)}
    if(i==2){myx<-c(1,0,0)}
    if(i==3){myx<-c(0,1,0)}
    if(i==4){myx<-c(0,0,1)}
    PY1.SZ1=PY1.SZ1+gvEAAR(Su,myx,alpha,beta)*fx.s2[,i]
    PY1.SZ0=PY1.SZ0+g_p(Su,myx,eta_p)*fx.s2[,i]
  }
  #in case PY1.SZ0 is very very small, and R recorded as 0:
  PY1.SZ0<-ifelse(PY1.SZ0==0,1e-300,PY1.SZ0)
  VE.S<-1-PY1.SZ1/PY1.SZ0
  output<-list(beta=beta,mu=mu,gamma=gamma,alpha=alpha,eta=eta,Su=Su, VE=VE.S)
  
  rm(VE.S)
  PY1.SZ0<-PY1.SZ1<-rep(0,length(Su))
  for (i in 1:length(Xu)){
    if(i==1){myx<-c(0,0,0)}
    if(i==2){myx<-c(1,0,0)}
    if(i==3){myx<-c(0,1,0)}
    if(i==4){myx<-c(0,0,1)}
    PY1.SZ1=PY1.SZ1+gvEAAR(Su,myx,alpha,beta)*fx.s2new[,i]
    PY1.SZ0=PY1.SZ0+g_p(Su,myx,eta_p)*fx.s2new[,i]
  }
  #in case PY1.SZ0 is very very small, and R recorded as 0:
  PY1.SZ0<-ifelse(PY1.SZ0==0,1e-300,PY1.SZ0)
  VE.S<-1-PY1.SZ1/PY1.SZ0
  output$VE.2=VE.S
  
  #calculate true VE
  alpha<-par.true[1:(2+length(levels(data$X))-1)]
  eta<-tail(par.true, 4+ 2*(length(levels(data$X))-1))
  eta_v<-c(eta[1]+eta[2],eta[3]+eta[4],eta[5]+eta[8] , eta[6]+eta[9], eta[7]+eta[10])
  eta_p<-c(eta[1],eta[3],eta[5] , eta[6], eta[7])
  
  ###
  
  fx.ss.true=cbind(fv.Su(c(0,0,0),Su,gamma.true),fv.Su(c(1,0,0),Su,gamma.true),fv.Su(c(0,1,0),Su,gamma.true),fv.Su(c(0,0,1),Su,gamma.true))
  #fX<-table(data[data$Z==1 & data$Ytau==0,]$X)/sum(!is.na(data[data$Z==1 & data$Ytau==0,]$X))  ## only to estimate fx empirically given the large sample size
  
  fx.s.true<-t(t(fx.ss.true)*as.numeric(fX))/apply(t(fx.ss.true)*as.numeric(fX),2,sum)
  
  fx.s2.true<-getfx.s2(fx.s.true,alpha,beta,eta_v)
  
  ####
  
  PY1.SZ0<-PY1.SZ1<-rep(0,length(Su))
  fx.s2<-getfx.s2(fx.s,alpha,beta,eta_v)
  for (i in 1:length(Xu)){
    if(i==1){myx<-c(0,0,0)}
    if(i==2){myx<-c(1,0,0)}
    if(i==3){myx<-c(0,1,0)}
    if(i==4){myx<-c(0,0,1)}
    PY1.SZ1=PY1.SZ1+gvEAAR(Su,myx,alpha,beta)*fx.s2[,i]
    PY1.SZ0=PY1.SZ0+g_p(Su,myx,eta_p)*fx.s2[,i]
  }
  #in case PY1.SZ0 is very very small, and R recorded as 0:
  PY1.SZ0<-ifelse(PY1.SZ0==0,1e-300,PY1.SZ0)
  VE.true<-1-PY1.SZ1/PY1.SZ0
  #rm(alpha,eta,eta_v,eta_p)
  output$VE.true<-VE.true
  ###
  
  rm(VE.true)
  PY1.SZ0<-PY1.SZ1<-rep(0,length(Su))
  #fx.s2<-getfx.s2(fx.s,alpha,beta,eta_v)
  for (i in 1:length(Xu)){
    if(i==1){myx<-c(0,0,0)}
    if(i==2){myx<-c(1,0,0)}
    if(i==3){myx<-c(0,1,0)}
    if(i==4){myx<-c(0,0,1)}
    PY1.SZ1=PY1.SZ1+gvEAAR(Su,myx,alpha,beta)*fx.s2.true[,i]
    PY1.SZ0=PY1.SZ0+g_p(Su,myx,eta_p)*fx.s2.true[,i]
  }
  #in case PY1.SZ0 is very very small, and R recorded as 0:
  PY1.SZ0<-ifelse(PY1.SZ0==0,1e-300,PY1.SZ0)
  VE.true<-1-PY1.SZ1/PY1.SZ0
  rm(alpha,eta,eta_v,eta_p)
  output$VE.true2<-VE.true
  
  
  return(output)
}


#### only difference with getVE is to enter a few starting values
getVE.sen<-function(data,beta,par.start,par.true,gamma.true,beta.true,Su,xi,par.start.2=NULL,par.start.3=NULL){
  #get nuisance parameters:
  #estimate \mu
  #mu.start <- c(4.6,rep(0,length(levels(data$X))-1))
  
  X.mu<-data.matrix(cbind(rep(1,nrow(data)),data[,grep("^[X]", names(data), value=TRUE)[-1]]))
  
  mu.start<-glm.fit(X.mu[data$Z==1,],1-data[data$Z==1,]$Ytau,weights=xi[data$Z==1], family=binomial(link="logit"))$coef
  #mu <- optim(par=mu.start, fn=logl1,data=data, xi=xi)$par
  mu <- mu.start
  rm(mu.start)
  
  #Estimate gamma
  #case sampling prob
  prob1<-sum( xi[ data$Ytau==0&data$Y==1&data$Z==1&!is.na(data$S1) ] )/sum(xi[data$Ytau==0&data$Z==1&data$Y==1])
  
  #control vaccinees sampling prob
  prob0<-sum(xi[data$Ytau==0&data$Y==0&data$Z==1&!is.na(data$S1)])/sum(xi[data$Ytau==0&data$Z==1&data$Y==0])
  
  #gamma.start <- c(2.4,rep(0,length(levels(data$X))-1), 0.58)
  
  ff<-ifelse(data$Y==1,(1/prob1),(1/prob0))
  kk=glm.fit(X.mu[data$Z==1 & data$Ytau==0 & !is.na(data$S),],data$S[data$Z==1 & data$Ytau==0 & !is.na(data$S)],weights=(ff*xi)[data$Z==1 & data$Ytau==0 & !is.na(data$S)],family=gaussian(link=identity))
  
  gamma.start<-c(kk$coef, sqrt(sum(kk$residuals^2*kk$weights)/sum(kk$weights)-sum((kk$residual*kk$weights)/sum(kk$weights))^2))
  #gamma <- optim(par=gamma.start, fn=logl2,prob0=prob0, prob1=prob1,data=data, xi=xi)$par
  gamma<-gamma.start
  rm(gamma.start)
  
  ### estimate eta
  out <- optim(par=par.start, fn=like,beta=beta,mu=mu,gamma=gamma,data=data, xi=xi )
  #par.start.1=par.start.2=par.start
  #par.start.1[1]=par.start.1[1]+1
  #par.start.2[2]=par.start.2[2]+0.5
  #out1 <- optim(par=par.start.1, fn=like,beta=beta,mu=mu,gamma=gamma,data=data, xi=xi)
  
  if (!is.null(par.start.2)) {
    out2 <- optim(par=par.start.2, fn=like,beta=beta,mu=mu,gamma=gamma,data=data, xi=xi)
    if (out2$value<out$value) out=out2
    rm(out2)
    }
  
  if (!is.null(par.start.3)) {
    out3 <- optim(par=par.start.3, fn=like,beta=beta,mu=mu,gamma=gamma,data=data, xi=xi)
    if (out3$value<out$value) out=out3
    rm(out3)
  }  
  
  out=out$par
  
  
  alpha<-out[1:(2+length(levels(data$X))-1)]
  eta<-tail(out, 4+ 2*(length(levels(data$X))-1))
  eta_v<-c(eta[1]+eta[2],eta[3]+eta[4],eta[5]+eta[8] , eta[6]+eta[9], eta[7]+eta[10])
  eta_p<-c(eta[1],eta[3],eta[5] , eta[6], eta[7])
  
  #######estimate X|S(1), Y\tau(0)=Y\tau(1)=0:
  Xu<-sort(unique(data$X[!is.na(data$X)]))
  datnew<-data.frame(S1=Su)
  
  
  #need to specify myx
  gvEAAR<-function(s,myx,alpha,beta){
    R<- alpha[1]+alpha[2]*s+alpha[3]*myx[1]+alpha[4]*myx[2]+alpha[5]*myx[3]+beta*1
    w_y1<-exp(R)/(1+exp(R))
    rm(R)
    
    R<- alpha[1]+alpha[2]*s+alpha[3]*myx[1]+alpha[4]*myx[2]+alpha[5]*myx[3]+beta*0
    w_y0<-exp(R)/(1+exp(R))
    rm(R)
    
    R<-eta_v[1]+eta_v[2]*s+eta_v[3]*myx[1]+eta_v[4]*myx[2]+eta_v[5]*myx[3]
    g_v<-pnorm(R)
    rm(R)
    
    gvEAAR <- w_y1*g_v/(w_y1*g_v+w_y0*(1-g_v))
    gvEAAR[gvEAAR==0]<-2e-10
    return(gvEAAR)
  }
  
  g_v<-function(s,myx,eta_v){
    R<-eta_v[1]+eta_v[2]*s+eta_v[3]*myx[1]+eta_v[4]*myx[2]+eta_v[5]*myx[3]
    return(pnorm(R))
  }
  
  g_p<-function(s,myx,eta_p){
    R<-eta_p[1]+eta_p[2]*s+eta_p[3]*myx[1]+eta_p[4]*myx[2]+eta_p[5]*myx[3]
    return(pnorm(R))
  }
  
  #P(Y^tau(0)=0|X=x, s_1, Y^tau(1)=0)
  f.Yyao0<-function(myx,s,alpha,beta,eta_v){
    R<- alpha[1]+alpha[2]*s+alpha[3]*myx[1]+alpha[4]*myx[2]+alpha[5]*myx[3]+beta*1
    w_y1<-exp(R)/(1+exp(R))
    rm(R)
    
    R<- alpha[1]+alpha[2]*s+alpha[3]*myx[1]+alpha[4]*myx[2]+alpha[5]*myx[3]+beta*0
    w_y0<-exp(R)/(1+exp(R))
    rm(R)
    
    return(w_y1*g_v(s,myx,eta_v)+w_y0*(1-g_v(s,myx,eta_v)))
  }
  
  ### compute P(X|S(1),Y^tau(0)=Y^tau(1)=0)
  getfx.s2<-function(fx.s,alpha,beta,eta_v){
  ## fx.s input is P(X|S(1),Y^tau(1)=0)
  
      fx.s2<-cbind(f.Yyao0(c(0,0,0),Su,alpha,beta,eta_v),f.Yyao0(c(1,0,0),Su,alpha,beta,eta_v),f.Yyao0(c(0,1,0),Su,alpha,beta,eta_v),f.Yyao0(c(0,0,1),Su,alpha,beta,eta_v))
      fx.s2<- fx.s*fx.s2
      den<-apply(fx.s2, 1, sum)
      den<-cbind(den,den,den,den)
      fx.s2<-fx.s2/den
      return(fx.s2)
  }
  
  # compute P(X=x|S(1),Y^tau(1)=0)
  #fX<-table(data$X)/sum(!is.na(data$X))
  data.delta1<-data[!is.na(data$S1),]
  
  #weights of sampling S1 
  data.delta1$weights<-rep(1,nrow(data.delta1))
  data.delta1$weights[data.delta1$Y==0]<-prob0
  
  wts <- 1/data.delta1$weights 
  wts<-wts*xi[!is.na(data$S1)]    #purturbation for estimating multinomial parameters
  
  fit=multinom(X~ns(S1,knots=quantile(Su,c(.25,.5,.75)),Boundary.knots=range(Su)),data=data.delta1,weights=wts)
  
  fx.s=predict(fit,datnew,type='probs')
  
  
  ## alternative way to get P(X=x|S(1),Y^tau(1)=0
  
  fv.Su<-function(myx,s,gamma){
    R<- gamma[1]+gamma[2]*myx[1]+gamma[3]*myx[2]+gamma[4]*myx[3]
    return(dnorm(s,R,gamma[5]))
  }
    
  fx.ss=cbind(fv.Su(c(0,0,0),Su,gamma),fv.Su(c(1,0,0),Su,gamma),fv.Su(c(0,1,0),Su,gamma),fv.Su(c(0,0,1),Su,gamma))
  #fX<-table(data[data$Z==1 & data$Ytau==0,]$X)/sum(!is.na(data[data$Z==1 & data$Ytau==0,]$X))
  
  fX<-tapply(xi[data$Z==1 & data$Ytau==0],data[data$Z==1 & data$Ytau==0,]$X,sum)/sum(xi[data$Z==1 & data$Ytau==0])
  
  
  
  fx.snew<-t(t(fx.ss)*as.numeric(fX))/apply(t(fx.ss)*as.numeric(fX),2,sum)
  
  
  fx.s2<-getfx.s2(fx.s,alpha,beta,eta_v)
  fx.s2new<-getfx.s2(fx.snew,alpha,beta,eta_v)
  PY1.SZ0<-PY1.SZ1<-rep(0,length(Su))
  for (i in 1:length(Xu)){
    if(i==1){myx<-c(0,0,0)}
    if(i==2){myx<-c(1,0,0)}
    if(i==3){myx<-c(0,1,0)}
    if(i==4){myx<-c(0,0,1)}
    PY1.SZ1=PY1.SZ1+gvEAAR(Su,myx,alpha,beta)*fx.s2[,i]
    PY1.SZ0=PY1.SZ0+g_p(Su,myx,eta_p)*fx.s2[,i]
  }
  #in case PY1.SZ0 is very very small, and R recorded as 0:
  PY1.SZ0<-ifelse(PY1.SZ0==0,1e-300,PY1.SZ0)
  VE.S<-1-PY1.SZ1/PY1.SZ0
  output<-list(beta=beta,mu=mu,gamma=gamma,alpha=alpha,eta=eta,Su=Su, VE=VE.S)
  
  rm(VE.S)
  PY1.SZ0<-PY1.SZ1<-rep(0,length(Su))
  for (i in 1:length(Xu)){
    if(i==1){myx<-c(0,0,0)}
    if(i==2){myx<-c(1,0,0)}
    if(i==3){myx<-c(0,1,0)}
    if(i==4){myx<-c(0,0,1)}
    PY1.SZ1=PY1.SZ1+gvEAAR(Su,myx,alpha,beta)*fx.s2new[,i]
    PY1.SZ0=PY1.SZ0+g_p(Su,myx,eta_p)*fx.s2new[,i]
  }
  #in case PY1.SZ0 is very very small, and R recorded as 0:
  PY1.SZ0<-ifelse(PY1.SZ0==0,1e-300,PY1.SZ0)
  VE.S<-1-PY1.SZ1/PY1.SZ0
  output$VE.2=VE.S
  
  #calculate true VE
  alpha<-par.true[1:(2+length(levels(data$X))-1)]
  eta<-tail(par.true, 4+ 2*(length(levels(data$X))-1))
  eta_v<-c(eta[1]+eta[2],eta[3]+eta[4],eta[5]+eta[8] , eta[6]+eta[9], eta[7]+eta[10])
  eta_p<-c(eta[1],eta[3],eta[5] , eta[6], eta[7])
  
  ###
  
  fx.ss.true=cbind(fv.Su(c(0,0,0),Su,gamma.true),fv.Su(c(1,0,0),Su,gamma.true),fv.Su(c(0,1,0),Su,gamma.true),fv.Su(c(0,0,1),Su,gamma.true))
  #fX<-table(data[data$Z==1 & data$Ytau==0,]$X)/sum(!is.na(data[data$Z==1 & data$Ytau==0,]$X))  ## only to estimate fx empirically given the large sample size
  
  fx.s.true<-t(t(fx.ss.true)*as.numeric(fX))/apply(t(fx.ss.true)*as.numeric(fX),2,sum)
  
  fx.s2.true<-getfx.s2(fx.s.true,alpha,beta.true,eta_v)
  
  ####
  
  PY1.SZ0<-PY1.SZ1<-rep(0,length(Su))
  fx.s2<-getfx.s2(fx.s,alpha,beta.true,eta_v)
  for (i in 1:length(Xu)){
    if(i==1){myx<-c(0,0,0)}
    if(i==2){myx<-c(1,0,0)}
    if(i==3){myx<-c(0,1,0)}
    if(i==4){myx<-c(0,0,1)}
    PY1.SZ1=PY1.SZ1+gvEAAR(Su,myx,alpha,beta.true)*fx.s2[,i]
    PY1.SZ0=PY1.SZ0+g_p(Su,myx,eta_p)*fx.s2[,i]
  }
  #in case PY1.SZ0 is very very small, and R recorded as 0:
  PY1.SZ0<-ifelse(PY1.SZ0==0,1e-300,PY1.SZ0)
  VE.true<-1-PY1.SZ1/PY1.SZ0
  #rm(alpha,eta,eta_v,eta_p)
  output$VE.true<-VE.true
  ###
  
  rm(VE.true)
  PY1.SZ0<-PY1.SZ1<-rep(0,length(Su))
  #fx.s2<-getfx.s2(fx.s,alpha,beta.true,eta_v)
  for (i in 1:length(Xu)){
    if(i==1){myx<-c(0,0,0)}
    if(i==2){myx<-c(1,0,0)}
    if(i==3){myx<-c(0,1,0)}
    if(i==4){myx<-c(0,0,1)}
    PY1.SZ1=PY1.SZ1+gvEAAR(Su,myx,alpha,beta.true)*fx.s2.true[,i]
    PY1.SZ0=PY1.SZ0+g_p(Su,myx,eta_p)*fx.s2.true[,i]
  }
  #in case PY1.SZ0 is very very small, and R recorded as 0:
  PY1.SZ0<-ifelse(PY1.SZ0==0,1e-300,PY1.SZ0)
  VE.true<-1-PY1.SZ1/PY1.SZ0
  rm(alpha,eta,eta_v,eta_p)
  output$VE.true2<-VE.true
  
  
  return(output)
}
